// enable text drag and insert
var textrect, textisDown, textorigX, textorigY;

const font_sizes = [1.333, 2.667, 4, 5.333, 6.667, 8, 9.333, 10.997, 12, 13.333, 14.667, 16, 18.667, 21.333, 24, 26.666, 32, 34.667, 37.333, 48, 64, 96,128];

const font_selection_data = [
    {
        id: "Montserrat",
        value: "Montserrat",
        text: 'Montserrat'
    },
    {
        id: "Arial",
        value: "Arial, Helvetica, sans-serif",
        text: 'Arial'
    },
    {
        id: "Times New Roman",
        value: "'Times New Roman', Times, serif",
        text: 'Times'
    },
    {
        id: "Courier",
        value: "Courier",
        text: 'Courier'
    },
    {
        id: "Comic Sans MS",
        value: "'Comic Sans MS', 'Comic Sans', cursive",
        text: "Comic Sans"
    },
];

const initTexts = () => {
    $("#update_text_font").select2({
        data: font_selection_data,
        templateResult: formatFontResults,
        minimumResultsForSearch: Infinity
    });
    $("#update_text_colour").on('change.spectrum', function(e, tinycolor) {
        console.log('updating colour')
        updateTextOnCanvas()
    });
}

const enableTexts = () => {
    // update cursor
    canvas.defaultCursor = 'text';
    canvas.selection = true;
    //canvas.setCursor('crosshair');
    canvas.on('mouse:down', textsOnMouseDown);
    canvas.on('mouse:move', textsOnMouseMove);
    canvas.on('mouse:up', textsOnMouseUp);
    lockAll();
    // hide the side panels
    //$(".selectionpanel").collapse('hide');
    $(".board-tool-section").hide();
    $(".top-buttons_right").css("display","");
    $(".text-tool-options").css("display","");
    $(".image_context_menu").css('display',"none");
}


const textsOnMouseDown = (event) => {
    textisDown = true;
    let pointer = canvas.getPointer(event.e);
    textorigX = pointer.x;
    textorigY = pointer.y;
    textrect = new fabric.Rect({
        left: textorigX,
        top: textorigY,
        originX: 'left',
        originY: 'top',
        width: pointer.x - textorigX,
        height: pointer.y - textorigY,
        angle: 0,
        fill: 'transparent',
        stroke: 'magenta',
        //fill: 'rgba(255,0,0,0.2)',
        transparentCorners: false
    });
    canvas.add(textrect);
}

const textsOnMouseMove = (event) => {
    if (!textisDown) return;
    let pointer = canvas.getPointer(event.e);
    if (textorigX > pointer.x) {
        textrect.set({left: Math.abs(pointer.x)});
    }
    if (textorigY > pointer.y) {
        textrect.set({top: Math.abs(pointer.y)});
    }
    textrect.set({width: Math.abs(textorigX - pointer.x)});
    textrect.set({height: Math.abs(textorigY - pointer.y)});
    canvas.renderAll();
}

const textsOnMouseUp = (event) => {
    unlockAll();
    canvas.off('mouse:down', textsOnMouseDown);
    canvas.off('mouse:move', textsOnMouseMove);
    canvas.off('mouse:up', textsOnMouseUp);
    canvas.defaultCursor = 'auto';
    textisDown = false;
    let textbox = new fabric.Textbox('Text', {
        left: textrect.left,
        top: textrect.top,
        width: textrect.width,
        height: textrect.height,
        fontSize: 18.667,
        fontStyle: ($("#update_text_italic").is(':checked') ? 'italic' : ''),
        fontWeight: ($("#update_text_bold").is(':checked') ? 'bold' : ''),
        underline: $("#update_text_underline").is(':checked'),
        textAlign: $("input[name='update_alignment']:checked").val(),
        fontFamily: $("#update_text_font").val()
    });
    canvas.remove(textrect)
    canvas.add(textbox);
    canvas.renderAll();
    textbox.enterEditing();
    canvas.setActiveObject(textbox);
    textbox.hiddenTextarea.focus();
    textbox.selectAll();
}

$("input[name='update_alignment']").change(function(){
    console.log("Changed");
})

// update the text on the canvas
const updateTextOnCanvas = () => {
    console.log("updateTextOnCanvas",$("input[name='update_alignment']:checked").val())
    let currentElement = canvas.getActiveObject();
    //console.log($("#update_text_font").val());
    currentElement.set('fontFamily', $("#update_text_font").val())
    currentElement.set('fill', $("#update_text_colour").spectrum('get').toString());
     currentElement.set('fontWeight', ($("#update_text_bold").is(':checked') ? 'bold' : ''));
     currentElement.set('fontStyle', ($("#update_text_italic").is(':checked') ? 'italic' : ''));
     currentElement.set('underline', $("#update_text_underline").is(':checked'));
     currentElement.set('textAlign', $("input[name='update_alignment']:checked").val())
    canvas.renderAll();
    $("input[name='update_alignment']").parent("button").removeClass("active");
    $("input[name='update_alignment']:checked").parent("button").addClass("active")
}

const increaseFont = () => {
    let currentElement = canvas.getActiveObject();
    if(currentElement) {
        let fontSize = currentElement.get('fontSize')
        // console.log(currentElement.get('fontSize'));
        let closest = font_sizes.reduce((a, b) => {
            return Math.abs(b - fontSize) < Math.abs(a - fontSize) ? b : a;
        });
        let current_size_index = font_sizes.findIndex((o) => {
            return o == closest;
        });
        console.log(closest);
        console.log(current_size_index);
        currentElement.set('fontSize', (current_size_index < (font_sizes.length - 1)) ? font_sizes[current_size_index + 1] : font_sizes[current_size_index]);
        canvas.renderAll();
    }
}

const decreaseFont = () => {
    let currentElement = canvas.getActiveObject();
    if(currentElement) {
        let fontSize = currentElement.get('fontSize')
        let closest = font_sizes.reduce((a, b) => {
            return Math.abs(b - fontSize) < Math.abs(a - fontSize) ? b : a;
        });
        let current_size_index = font_sizes.findIndex((o) => {
            return o == closest;
        });
        currentElement.set('fontSize', (current_size_index == 0) ? font_sizes[current_size_index] : font_sizes[current_size_index - 1]);
        canvas.renderAll();
    }
}

//Custom formatting function for font picker select2 component
const formatFontResults = (item) => {
    let $label = $(
        '<span style="font-family: ' + item.value + '">' + item.text + '</span>'
    );
    return $label;
};
